# Keras README

## Usage

### Extract Keras model

We provide an example code *mmdnn/conversion/examples/keras/extract_model.py* to extract Keras pre-trained model. You can refer it to extract your own model.

> usage: extract_model.py [-h] -n {vgg16,vgg19,inception_v3,resnet50,mobilenet,xception}

Simplify said, use following Python code to extract network architecture
```python
json_string = model.to_json()
with open("model_architecture.json", "w") as of:
    of.write(json_string)
```

and Python code following to extract network weights
```python
model.save_weights('model_weights.h5')
```

### Convert architecture from Keras to IR

You can convert only network structure to IR for visualization or training in other frameworks.

```bash
$ python -m mmdnn.conversion._script.convertToIR -f keras -d inception_v3 -n imagenet_inceptionv3.json

Using TensorFlow backend.
IR network structure is saved as [inception_v3.json].
IR network structure is saved as [inception_v3.pb].
Warning: weights are not loaded.
```



### Convert model (including architecture and weights) from Keras to IR

You can use following bash command to convert the network architecture [*imagenet_inceptionv3.json*] with weights [*imagenet_inceptionv3.h5*] to IR architecture file [*inception_v3.pb*], [*inception_v3.json*] and IR weights file [*inception_v3.npy*]

```bash
$ python -m mmdnn.conversion._script.convertToIR -f keras -d inception_v3 -n imagenet_inceptionv3.json -w imagenet_inceptionv3.h5

Using TensorFlow backend.
.
.
.
Network file [imagenet_inceptionv3.json] is loaded successfully.
IR network structure is saved as [inception_v3.json].
IR network structure is saved as [inception_v3.pb].
IR weights are saved as [inception_v3.npy].
```

### Convert models from IR to Keras code snippet

Since the generated Keras model code snippet can restore weights from IR weights file directly, we don't need to convert weights in this step.

```bash
$ python -m mmdnn.conversion._script.IRToCode -f keras --IRModelPath inception_v3.pb --dstModelPath keras_inception_v3.py

Parse file [inception_v3.pb] with binary format successfully.
Target network code snippet is saved as [keras_inception_v3.py].
```

### Convert models from IR to Keras model

After generating the keras code snippet, you can convert the Keras code snippet and IR weights file to Keras original model for further usage.

```bash
$ python -m mmdnn.conversion.examples.keras.imagenet_test -n keras_inception_v3.py -w inception_v3.npy --dump keras_inception_v3.h5

Using TensorFlow backend.
.
.
.
Keras model file is saved as [keras_inception_v3.h5], generated by [keras_inception_v3.py] and [inception_v3.npy].
```

## Support Operators

- Add
- Concat
- Relu
- Softmax
- Tanh
- Sigmoid
- Softplus
- Softsign
- HardSigmoid
- Elu

## Develop version

Ubuntu 16.04 with

- Keras 2.1.1

- Tensorflow GPU 1.4.0

@ 11/21/2017

## Limitation

- Lambda layer no support

  - *inception_resnet_v2* pre-trained model no support

- Currently no RNN related operations support

- Model in model is not supported yet. We can implement it if needed.
